#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "dialogabout.h"
#include <QFileDialog>
#include <QDesktopServices>
#include <QMessageBox>
#include <QTime>
#include <QClipboard>
#include <QMimeData>
#include <QColorDialog>
#include <QTextList>
#include <QTextDocumentWriter>
#include <QTextCodec>
#include <QPrinter>
#include <QPrintDialog>
#include <QPrintPreviewDialog>
#include <QCloseEvent>
#include <QSettings>
MainWindow::MainWindow(QWidget *parent) :
        QMainWindow(parent),
        ui(new Ui::MainWindow)
{
        ui->setupUi(this);


        QAction *actDock=ui->dockWidget->toggleViewAction();
        actDock->setIcon(QIcon(":/images/elforkane.png"));
        ui->mainToolBar->addAction(actDock);
        ui->menuEdit->addAction(actDock);
        ui->dockWidget->toggleViewAction()->setShortcut(QKeySequence(tr("Ctrl+Shift+I")));
        ui->dockWidget->setVisible(false);
        connect(ui->actionPlayPause, SIGNAL(triggered()), this, SLOT(playPause()));
        ui->pButtonPlayPause->setDefaultAction(ui->actionPlayPause);
        connect(ui->actionStop, SIGNAL(triggered()), &m_MediaObject, SLOT(stop()));
        ui->pButtonStop->setDefaultAction(ui->actionStop);

        connect(ui->actionBackward, SIGNAL(triggered()), this, SLOT(mediaBackward()));
        ui->tButtonBackward->setDefaultAction(ui->actionBackward);
        connect(ui->actionForward, SIGNAL(triggered()), this, SLOT(mediaForward()));
        ui->tButtonForward->setDefaultAction(ui->actionForward);
        connect(ui->actionGo, SIGNAL(triggered()), this, SLOT(mediaGoPos()));
        ui->tButtonGo->setDefaultAction(ui->actionGo);
        connect(ui->actionOpenMedia, SIGNAL(triggered()), this, SLOT(openMedias()));
        ui->tButtonAddMedia->setDefaultAction(ui->actionOpenMedia);

        connect(&m_MediaObject, SIGNAL(stateChanged(Phonon::State,Phonon::State)), this, SLOT(stateChanged(Phonon::State,Phonon::State)));
        connect(&m_MediaObject, SIGNAL(totalTimeChanged(qint64)), this, SLOT(updateTime()));
        connect(&m_MediaObject, SIGNAL(tick(qint64)), this, SLOT(updateTime()));
        connect(&m_MediaObject, SIGNAL(metaDataChanged()), this, SLOT(updateInfo()));
        connect(&m_MediaObject, SIGNAL(hasVideoChanged(bool)), this, SLOT(hasVideoChanged(bool)));

        //  connect(m_videoWidget, SIGNAL(customContextMenuRequested(const QPoint &)), SLOT(showContextMenu(const QPoint &)));
        Phonon::createPath(&m_MediaObject, &m_AudioOutput);
        Phonon::createPath(&m_MediaObject, ui->VideoWidget);


        ui->seekSlider->setMediaObject(&m_MediaObject);
        //ui->volumeSlider->setAudioOutput(&m_AudioOutput);
        ui->VideoWidget->setVisible(false);

        //------------text edite--------------------------
        QActionGroup *grp = new QActionGroup(this);
        connect(grp, SIGNAL(triggered(QAction*)), this, SLOT(textAlign(QAction*)));
        actionAlignRight = new QAction(trUtf8("يمين"), grp);
        actionAlignCenter = new QAction(trUtf8("توسيط"), grp);
        actionAlignLeft = new QAction(trUtf8("يسار"), grp);
        actionAlignJustify = new QAction(trUtf8("السياق"), grp);

        actionAlignLeft->setCheckable(true);
        actionAlignLeft->setPriority(QAction::LowPriority);
        actionAlignCenter->setCheckable(true);
        actionAlignCenter->setPriority(QAction::LowPriority);
        actionAlignRight->setCheckable(true);
        actionAlignRight->setPriority(QAction::LowPriority);
        actionAlignJustify->setCheckable(true);
        actionAlignJustify->setPriority(QAction::LowPriority);

        ui->toolBarStyle->addActions(grp->actions());
        ui->menuFormat->addActions(grp->actions());
        //قائمة منسدلة بالطراز
        comboStyle = new QComboBox(ui->toolBarStyle);
        ui->toolBarStyle->addWidget(comboStyle);
        comboStyle->addItem("Standard");
        comboStyle->addItem("Bullet List (Disc)");
        comboStyle->addItem("Bullet List (Circle)");
        comboStyle->addItem("Bullet List (Square)");
        comboStyle->addItem("Ordered List (Decimal)");
        comboStyle->addItem("Ordered List (Alpha lower)");
        comboStyle->addItem("Ordered List (Alpha upper)");
        comboStyle->addItem("Ordered List (Roman lower)");
        comboStyle->addItem("Ordered List (Roman upper)");
        connect(comboStyle, SIGNAL(activated(int)),
                this, SLOT(textStyle(int)));
        //قائمة منسدلة بالخطوط
        comboFont = new QFontComboBox(ui->toolBarStyle);
        ui->toolBarStyle->addWidget(comboFont);
        connect(comboFont, SIGNAL(activated(QString)),
                this, SLOT(textFamily(QString)));

        //قائمة منسدلة باحجام الخطوط

        comboSize = new QComboBox(ui->toolBarStyle);
        comboSize->setObjectName("comboSize");
        ui->toolBarStyle->addWidget(comboSize);
        comboSize->setEditable(true);
        QFontDatabase db;
        foreach(int size, db.standardSizes())
                comboSize->addItem(QString::number(size));

        connect(comboSize, SIGNAL(activated(QString)),
                this, SLOT(textSize(QString)));
        comboSize->setCurrentIndex(comboSize->findText(QString::number(QApplication::font()
                                                                       .pointSize())));

        //--------------widgetQuran *widQuran;
        widQuran=new widgetQuran;
        ui->verticalLayoutQuran->addWidget(widQuran);
        //---------------------connect actions---------------------
        connect(widQuran, SIGNAL(inserText(QString)), this, SLOT(setTextQuran(QString)));
        connect(widQuran, SIGNAL(inserSymbole(QString)), this, SLOT(setTextSymbole(QString)));
        connect(ui->actionNew, SIGNAL(triggered()), this, SLOT(fileNew()));

        connect(ui->actionSave, SIGNAL(triggered()), this, SLOT(fileSave()));
        ui->actionSave->setEnabled(false);

        connect(ui->actionSaveAs, SIGNAL(triggered()), this, SLOT(fileSaveAs()));


#ifndef QT_NO_PRINTER
        connect(ui->actionPrint, SIGNAL(triggered()), this, SLOT(filePrint()));
        connect(ui->actionPrintPreview, SIGNAL(triggered()), this, SLOT(filePrintPreview()));
        connect(ui->actionExportPdf, SIGNAL(triggered()), this, SLOT(filePrintPdf()));
#endif

        connect(ui->actionQuit, SIGNAL(triggered()), this, SLOT(close()));



        connect(ui->actionColor, SIGNAL(triggered()), this, SLOT(textColor()));

        connect(ui->textEdit, SIGNAL(currentCharFormatChanged(QTextCharFormat)),
                this, SLOT(currentCharFormatChanged(QTextCharFormat)));
        connect(ui->textEdit, SIGNAL(cursorPositionChanged()),
                this, SLOT(cursorPositionChanged()));

        connect(ui->textEdit->document(), SIGNAL(modificationChanged(bool)),
                ui->actionSave, SLOT(setEnabled(bool)));
        connect(ui->textEdit->document(), SIGNAL(modificationChanged(bool)),
                this, SLOT(setWindowModified(bool)));
        connect(ui->textEdit->document(), SIGNAL(undoAvailable(bool)),
                ui->actionUndo, SLOT(setEnabled(bool)));
        connect(this, SIGNAL(mediaChanged(bool)),
                ui->textEdit->document(), SIGNAL(modificationChanged(bool)));

        connect(ui->textEdit->document(), SIGNAL(redoAvailable(bool)),
                ui->actionRedo, SLOT(setEnabled(bool)));

        connect(ui->actionUndo, SIGNAL(triggered()), ui->textEdit, SLOT(undo()));
        connect(ui->actionRedo, SIGNAL(triggered()), ui->textEdit, SLOT(redo()));

        connect(ui->actionCut, SIGNAL(triggered()), ui->textEdit, SLOT(cut()));
        connect(ui->actionCopy, SIGNAL(triggered()), ui->textEdit, SLOT(copy()));
        connect(ui->actionPaste, SIGNAL(triggered()), ui->textEdit, SLOT(paste()));

        connect(ui->textEdit, SIGNAL(copyAvailable(bool)), ui->actionCut, SLOT(setEnabled(bool)));
        connect(ui->textEdit, SIGNAL(copyAvailable(bool)), ui->actionCopy, SLOT(setEnabled(bool)));

        connect(ui->actionBold, SIGNAL(triggered()), this, SLOT(textBold()));
        connect( ui->actionItalic, SIGNAL(triggered()), this, SLOT(textItalic()));
        connect(ui->actionUnderline, SIGNAL(triggered()), this, SLOT(textUnderline()));
        connect(ui->actionInsertBookmark, SIGNAL(triggered()), this, SLOT(addBokmark()));
        connect(ui->actionAbout, SIGNAL(triggered()), this, SLOT(aboutThisProgram()));


        //----------------------Shortcuts------------------------------
        actionAlignLeft->setShortcut(Qt::CTRL + Qt::Key_L);
        actionAlignCenter->setShortcut(Qt::CTRL + Qt::Key_E);
        actionAlignRight->setShortcut(Qt::CTRL + Qt::Key_R);
        actionAlignJustify->setShortcut(Qt::CTRL + Qt::Key_J);


        ui->actionNew->setShortcut(QKeySequence::New);
        ui->actionSave->setShortcut(QKeySequence::Save);
        ui->actionPrint->setShortcut(QKeySequence::Print);
        ui->actionExportPdf->setShortcut(Qt::CTRL + Qt::Key_D);
        ui->actionQuit->setShortcut(Qt::CTRL + Qt::Key_Q);

        ui->actionBold->setShortcut(Qt::CTRL + Qt::Key_B);
        ui->actionItalic->setShortcut(Qt::CTRL + Qt::Key_I);
        ui->actionUnderline ->setShortcut(Qt::CTRL + Qt::Key_U);

        ui->actionUndo->setShortcut(QKeySequence::Undo);
        ui->actionRedo->setShortcut(QKeySequence::Redo);

        ui->actionCut->setShortcut(QKeySequence::Cut);
        ui->actionCopy->setShortcut(QKeySequence::Copy);
        ui->actionPaste->setShortcut(QKeySequence::Paste);

        ui->actionBackward->setShortcut(Qt::Key_F5);
        ui->actionForward->setShortcut(Qt::Key_F6);
        ui->actionStop->setShortcut(Qt::CTRL +Qt::Key_S);
        //ui->actionPlayPause->setShortcut(Qt::CTRL +Qt::Backs);
        ui->actionGo->setShortcut(Qt::Key_F7);
        ui->actionInsertBookmark->setShortcut(Qt::Key_F8);
        //----------------------------------------------------

        actionAlignLeft->setStatusTip(actionAlignLeft->text()+"   "+actionAlignLeft->shortcut().toString());
        actionAlignCenter->setStatusTip(actionAlignCenter->text()+"   "+actionAlignCenter->shortcut().toString());
        actionAlignRight->setStatusTip(actionAlignRight->text()+"   "+actionAlignRight->shortcut().toString());
        actionAlignJustify->setStatusTip(actionAlignJustify->text()+"   "+actionAlignJustify->shortcut().toString());


        ui->actionNew->setStatusTip(ui->actionNew->text()+"   "+ui->actionNew->shortcut().toString());
        ui->actionSave->setStatusTip(ui->actionSave->text()+"   "+ui->actionSave->shortcut().toString());
        ui->actionPrint->setStatusTip(ui->actionPrint->text()+"   "+ui->actionPrint->shortcut().toString());
        ui->actionExportPdf->setStatusTip(ui->actionExportPdf->text()+"   "+ui->actionExportPdf->shortcut().toString());
        ui->actionQuit->setStatusTip(ui->actionQuit->text()+"   "+ui->actionQuit->shortcut().toString());

        ui->actionBold->setStatusTip(ui->actionBold->text()+"   "+ui->actionBold->shortcut().toString());
        ui->actionItalic->setStatusTip(ui->actionItalic->text()+"   "+ui->actionItalic->shortcut().toString());
        ui->actionUnderline ->setStatusTip(ui->actionUnderline->text()+"   "+ui->actionUnderline->shortcut().toString());

        ui->actionUndo->setStatusTip(ui->actionUndo->text()+"   "+ui->actionUndo->shortcut().toString());
        ui->actionRedo->setStatusTip(ui->actionRedo->text()+"   "+ui->actionRedo->shortcut().toString());

        ui->actionCut->setStatusTip(ui->actionCut->text()+"   "+ui->actionCut->shortcut().toString());
        ui->actionCopy->setStatusTip(ui->actionCopy->text()+"   "+ui->actionCopy->shortcut().toString());
        ui->actionPaste->setStatusTip(ui->actionPaste->text()+"   "+ui->actionPaste->shortcut().toString());

        ui->actionBackward->setStatusTip(ui->actionBackward->text()+"   "+ui->actionBackward->shortcut().toString());
        ui->actionForward->setStatusTip(ui->actionForward->text()+"   "+ui->actionForward->shortcut().toString());
        ui->actionStop->setStatusTip(ui->actionStop->text()+"   "+ui->actionStop->shortcut().toString());
        //ui->actionPlayPause->setShortcut(Qt::CTRL +Qt::Backs);
        ui->actionGo->setStatusTip(ui->actionGo->text()+"   "+ui->actionGo->shortcut().toString());
        ui->actionInsertBookmark->setStatusTip(ui->actionInsertBookmark->text()+"   "+ui->actionInsertBookmark->shortcut().toString());

        ui->actionBackward->setToolTip(ui->actionBackward->text()+"   "+ui->actionBackward->shortcut().toString());
        ui->actionForward->setToolTip(ui->actionForward->text()+"   "+ui->actionForward->shortcut().toString());
        ui->actionStop->setToolTip(ui->actionStop->text()+"   "+ui->actionStop->shortcut().toString());
        //ui->actionPlayPause->setShortcut(Qt::CTRL +Qt::Backs);
        ui->actionGo->setToolTip(ui->actionGo->text()+"   "+ui->actionGo->shortcut().toString());
        ui->actionInsertBookmark->setToolTip(ui->actionInsertBookmark->text()+"   "+ui->actionInsertBookmark->shortcut().toString());




        themeIcons();
        //       setCurrentFileName(QString());

        fontChanged(ui->textEdit->font());
        colorChanged(ui->textEdit->textColor());
        alignmentChanged(ui->textEdit->alignment());
        setWindowModified(ui->textEdit->document()->isModified());
#ifndef QT_NO_CLIPBOARD
        if (const QMimeData *md = QApplication::clipboard()->mimeData())
                ui->actionPaste->setEnabled(md->hasText());
#endif
        ui->actionSave->setEnabled(ui->textEdit->document()->isModified());
        ui->actionUndo->setEnabled(ui->textEdit->document()->isUndoAvailable());
        ui->actionRedo->setEnabled(ui->textEdit->document()->isRedoAvailable());
        ui->actionCut->setEnabled(false);
        ui->actionCopy->setEnabled(false);
        ui->textEdit->setFocus();
        load("");
}

MainWindow::~MainWindow()
{
        QSettings setting("textextract","settings");
        setting.beginGroup("Geometry");
        setting.setValue("geo_data", saveGeometry());
        setting.setValue("layout_data", saveState(0));
       setting.endGroup();
        delete ui;
}
void MainWindow::closeEvent(QCloseEvent *e)
{
        if (maybeSave())
                e->accept();
        else
                e->ignore();
}

void MainWindow::themeIcons()
{
        //phonon icons---------------------------------------------------------------------------
        if(isRightToLeft()){
                ui->actionForward->setIcon(QIcon::fromTheme("go-first",QIcon(":/images/go-first.png")));
                ui->actionBackward->setIcon(QIcon::fromTheme("go-last",QIcon(":/images/go-last.png")));
        }else{
                ui->actionBackward->setIcon(QIcon::fromTheme("go-first",QIcon(":/images/go-first.png")));
                ui->actionForward->setIcon(QIcon::fromTheme("go-last",QIcon(":/images/go-last.png")));

        }
        m_pauseIcon=     QIcon::fromTheme("media-playback-pause",QIcon(":/images/media-playback-pause.png"));
        m_playIcon=QIcon::fromTheme("media-playback-start",QIcon(":/images/media-playback-start.png"));
        ui->actionPlayPause->setIcon(m_playIcon);
        ui->actionStop->setIcon(QIcon::fromTheme("media-playback-stop",QIcon(":/images/media-playback-stop.png")));
        ui->actionGo->setIcon(QIcon::fromTheme("go-bottom",QIcon(":/images/go-pos.png")));
        ui->actionOpenMedia->setIcon(QIcon::fromTheme("document-open",QIcon(":/images/document-open.png")));
        //text edit icons--------------------------------------------------------------------------------------
        //        QPixmap pix(16, 16);
        //        pix.fill(Qt::black);
        //       ui->actionColor->setIcon(pix);

        ui->actionUndo->setIcon(QIcon::fromTheme("edit-undo", QIcon( ":/images/edit-undo.png")));
        ui->actionRedo->setIcon(QIcon::fromTheme("edit-redo", QIcon( ":/images/edit-redo.png")));
        ui->actionCut->setIcon(QIcon::fromTheme("edit-cut", QIcon( ":/images/edit-cut.png")));
        ui->actionCopy->setIcon(QIcon::fromTheme("edit-copy", QIcon( ":/images/edit-copy.png")));
        ui->actionPaste->setIcon(QIcon::fromTheme("edit-paste", QIcon( ":/images/edit-paste.png")));

        ui->actionUnderline ->setIcon(QIcon::fromTheme("format-text-underline", QIcon(":/images/format-text-underline.png")));
        ui->actionItalic->setIcon(QIcon::fromTheme("format-text-italic", QIcon( ":/images/format-text-italic.png")));
        ui->actionBold->setIcon(QIcon::fromTheme("format-text-bold", QIcon( ":/images/format-text-bold.png")));

        // Make sure the alignLeft  is always left of the alignRight
        if (QApplication::isLeftToRight()) {
                actionAlignLeft->setIcon(QIcon::fromTheme("format-justify-left", QIcon(":/images/format-justify-left.png")) );
                actionAlignRight->setIcon(QIcon::fromTheme("format-justify-right", QIcon(":/images/format-justify-right.png")));
        } else {
                actionAlignRight ->setIcon(QIcon::fromTheme("format-justify-right", QIcon(":/images/format-justify-right.png")));
                actionAlignLeft->setIcon(QIcon::fromTheme("format-justify-left", QIcon( ":/images/format-justify-left.png")));
        }
        actionAlignJustify ->setIcon(QIcon::fromTheme("format-justify-fill", QIcon( ":/images/format-justify-fill.png")));
        actionAlignCenter ->setIcon(QIcon::fromTheme("format-justify-center", QIcon(":/images/format-justify-center.png")));

        ui->actionNew->setIcon(QIcon::fromTheme("document-new", QIcon(  ":/images/document-new.png")));
        ui->actionSave->setIcon(QIcon::fromTheme("document-save", QIcon(":/images/document-save.png")));
        ui->actionSaveAs->setIcon(QIcon::fromTheme("document-save-as", QIcon(":/images/document-save-as.png")));
        ui->actionPrint->setIcon(QIcon::fromTheme("document-print", QIcon( ":/images/document-print.png")));
        ui->actionPrintPreview->setIcon(QIcon::fromTheme("fileprint", QIcon(":/images/document-print.png")));
        ui->actionQuit->setIcon(QIcon::fromTheme("gtk-quit", QIcon(":/images/gtk-quit.png")));
        ui->actionExportPdf->setIcon(QIcon::fromTheme("pdf", QIcon(":/images/pdf.png")));
        ui->actionInsertBookmark->setIcon(QIcon::fromTheme("bookmark_add", QIcon(":/images/bookmark_add.png")));
}
/**********************************************************************************
 ****************                    PHONON              ***************************
 ***********************************************************************************/
//-----------------فتح ملف الوسائط
void MainWindow::openMedias()
{
        QStringList fileNS = QFileDialog::getOpenFileNames(this, QString(),
                                                           QDesktopServices::storageLocation(QDesktopServices::MusicLocation));
        if (fileNS.count() > 0) {
                recentPos=0;
                ui->comboBox->clear();
                //fileName= fileN;
                ui->comboBox->addItems(fileNS);
                soundFiles=fileNS;
                mediaSetSource(ui->comboBox->currentText());
                ui->textEdit->document()->setModified(true);
        }
}
//-----------تغير مسار ملف الوسيط---------------------------------
void MainWindow::on_comboBox_activated(const QString &arg1)
{
        recentPos=0;
        mediaSetSource(arg1);
}

void MainWindow::mediaSetSource(const QString &arg1)
{
        ui->VideoWidget->setVisible(false);
        m_MediaObject.setCurrentSource(Phonon::MediaSource(arg1));
        m_MediaObject.pause();
}
//-------معرفة حالة الوسيط-----------------------------------------

void MainWindow::stateChanged(Phonon::State newstate, Phonon::State )
{
        switch (newstate) {
        case Phonon::ErrorState:
                if (m_MediaObject.errorType() == Phonon::FatalError) {
                        ui->pButtonPlayPause->setEnabled(false);
                        //rewindButton->setEnabled(false);
                } else {
                        m_MediaObject.pause();
                }
                QMessageBox::warning(this, "Phonon Mediaplayer", m_MediaObject.errorString(), QMessageBox::Close);
                break;

        case Phonon::StoppedState:
                //  m_videoWidget->setFullScreen(false);
                // Fall through
        case Phonon::PausedState:
                ui->pButtonPlayPause->setIcon(m_playIcon);
                if (m_MediaObject.currentSource().type() != Phonon::MediaSource::Invalid){
                        ui->pButtonPlayPause->setEnabled(true);
                        //rewindButton->setEnabled(true);
                } else {
                        ui->pButtonPlayPause->setEnabled(false);
                        // rewindButton->setEnabled(false);
                }
                break;
        case Phonon::PlayingState:
                ui->pButtonPlayPause->setEnabled(true);
                ui->pButtonPlayPause->setIcon(m_pauseIcon);
                if (m_MediaObject.hasVideo())
                        //m_videoWidget.show();
                        // Fall through
        case Phonon::BufferingState:
                        //rewindButton->setEnabled(true);
                        break;
                case Phonon::LoadingState:
                        // rewindButton->setEnabled(false);
                        break;
        }
}
//---------تحديث المغلومات-----------------------
void MainWindow::updateTime()
{
        long len = m_MediaObject.totalTime();
        long pos = m_MediaObject.currentTime();
        QString timeString;

        if (pos || len)
        {
                int sec = pos/1000;
                int min = sec/60;
                int hour = min/60;
                int msec = pos;

                QTime playTime(hour%60, min%60, sec%60, msec%1000);
                sec = len / 1000;
                min = sec / 60;
                hour = min / 60;
                msec = len;

                QTime stopTime(hour%60, min%60, sec%60, msec%1000);
                QString timeFormat = "m:ss";
                if (hour > 0)
                        timeFormat = "h:mm:ss";
                timeString = playTime.toString(timeFormat);
                if (len)
                        timeString += " / " + stopTime.toString(timeFormat);
                //   ui->timeEditBackForward->setMaximumTime(stopTime);
                ui->timeEdit->setMaximumTime(stopTime);
                //   ui->timeEdit->setTime(playTime);
        }

        ui->labelTime->setText(timeString);
        //ui->spinBoxSeek->setValue(pos/1000);
        //  ui->labelCurTime->setText(QString::number(pos/1000));
}
//--------------تشغيل توقف----------------------
void MainWindow::playPause()
{
        if (m_MediaObject.state() == Phonon::PlayingState)
                m_MediaObject.pause();
        else {
                if(recentPos!=0){
                        m_MediaObject.seek(recentPos);
                        recentPos=0;
                }
                if (m_MediaObject.currentTime() == m_MediaObject.totalTime())
                        m_MediaObject.seek(0);
                m_MediaObject.play();
        }
}
//--------------الذهاب للخلف-------------------
void MainWindow::mediaBackward()
{


        QTime time=ui->timeEditBackForward->time();
        int sec = time.second()*1000;
        int min = time.minute()*60000;
        int hour = time.hour()*3600000;
        int msec = sec+min+hour;

        qDebug()<< msec;
        qint64 value= (m_MediaObject.currentTime())-msec;
        if(value<0)return;
        m_MediaObject.seek(value);
}
//-----التنقل للامام----------------------------
void MainWindow::mediaForward()
{
        QTime time=ui->timeEditBackForward->time();
        int sec = time.second()*1000;
        int min = time.minute()*60000;
        int hour = time.hour()*3600000;
        int msec = sec+min+hour;
        qint64 value= (m_MediaObject.currentTime())+msec;
        if(value>m_MediaObject.totalTime())return;
        m_MediaObject.seek(value);
}

//-----------الذهاب الى موضع--------------------
void MainWindow::mediaGoPos()
{
        QTime time=ui->timeEdit->time();
        int sec = time.second()*1000;
        int min = time.minute()*60000;
        int hour = time.hour()*3600000;
        int msec = sec+min+hour;
        qint64 value= (msec);

        m_MediaObject.seek(value);
}
//------------في حالة تشغيل فيديو----------------
void MainWindow::hasVideoChanged(bool bHasVideo)
{
        qDebug()<<"bHasVideo : "<<bHasVideo;
        ui->VideoWidget->setVisible(bHasVideo);
        // m_fullScreenAction->setEnabled(bHasVideo);
}

/**********************************************************************************
 ***********************       TEXT EDITOR         ********************************
 ***********************************************************************************/
void MainWindow::fileNew()
{
        if (maybeSave()) {
                ui->textEdit->clear();
                ui->comboBox->clear();
                m_MediaObject.stop();
                m_MediaObject.clear();
                setCurrentFileName(QString());
        }

}
void MainWindow::setCurrentFileName(const QString &fileName)
{
        this->fileName = fileName;
        ui->textEdit->document()->setModified(false);

        QString shownName;
        if (fileName.isEmpty())
                shownName = "untitled.txt";
        else
                shownName = QFileInfo(fileName).fileName();

        setWindowTitle(tr("%1[*] - %2").arg(shownName).arg(tr("Rich Text")));
        setWindowModified(false);
}
void MainWindow::fileOpen()
{
        QString fn = QFileDialog::getOpenFileName(this, tr("Open File..."),
                                                  QString(), tr("HTML-Files (*.htm *.html);;All Files (*)"));
        if (!fn.isEmpty())
                load(fn);
}
bool MainWindow::load(const QString &f)
{
        recentPos=0;
        QSettings setting("textextract","settings");
        setting.beginGroup("Geometry");
       this->restoreGeometry(setting.value("geo_data").toByteArray());
       this->restoreState( setting.value("layout_data").toByteArray());
       setting.endGroup();
        setting.beginGroup("Sound-Path");
        fileName= setting.value("fileName",QString()).toString();
        soundFiles=     setting.value("fileList").toStringList();
        QString curentMedia=    setting.value("CurentFile","").toString();
        recentPos= setting.value("CurentPos",0).toInt();
         setting.endGroup();

        ui->comboBox->addItems(soundFiles);
        int idx=ui->comboBox->findText(curentMedia);
        ui->comboBox->setCurrentIndex(idx);
        mediaSetSource(curentMedia);


   setCurrentFileName(fileName);
        if (!QFile::exists(fileName)){
                setCurrentFileName(QString());
                return false;
        }
        QFile file(fileName);
        if (!file.open(QFile::ReadOnly))
                return false;

        QByteArray data = file.readAll();
        QTextCodec *codec = Qt::codecForHtml(data);
        QString str = codec->toUnicode(data);
        if (Qt::mightBeRichText(str)) {
                ui->textEdit->setHtml(str);
        } else {
                str = QString::fromLocal8Bit(data);
                ui->textEdit->setPlainText(str);
        }


        return true;
}

bool MainWindow::maybeSave()
{
        if (!ui->textEdit->document()->isModified())
                return true;
        if (fileName.startsWith(QLatin1String(":/")))
                return true;
        QMessageBox::StandardButton ret;
        ret = QMessageBox::warning(this, trUtf8(""),
                                   trUtf8("لقد تم التعديل على الملف.\n"
                                      "هل تريد حفظ التغييرات ؟"),
                                   QMessageBox::Save | QMessageBox::Discard
                                   | QMessageBox::Cancel);
        if (ret == QMessageBox::Save)
                return fileSave();
        else if (ret == QMessageBox::Cancel)
                return false;
        return true;
}

bool MainWindow::fileSave()
{
       qDebug()<<fileName;
        if (fileName.isEmpty())
            return fileSessionSaveAs();

        QTextDocumentWriter writer(fileName);
        bool success = writer.write(ui->textEdit->document());
        if (success)
            ui->textEdit->document()->setModified(false);
     //   return success;

        QSettings setting("textextract","settings");

        setting.beginGroup("Sound-Path");
        setting.setValue("fileName",fileName);
        setting.setValue("fileList",soundFiles);
        setting.setValue("CurentFile",ui->comboBox->currentText());
        setting.setValue("CurentPos",m_MediaObject.currentTime());
        setting.endGroup();
        qDebug()<<m_MediaObject.currentTime();
        QDir dir(QDir::homePath())   ;
        dir.mkdir(".textextract");
//        QTextDocumentWriter writer(QDir::homePath()+"/.textextract/unsaved.html");
//        setCurrentFileName(QDir::homePath()+"/.textextract/unsaved.html");
//        bool success = writer.write(ui->textEdit->document());
//        if (success)
//                ui->textEdit->document()->setModified(false);
        return success;
}
bool MainWindow::fileSessionSaveAs()
{
        QString fn = QFileDialog::getSaveFileName(this, tr("Save as..."),
                                                  QString(), tr("HTML-Files (*.htm *.html)"));
        if (fn.isEmpty())
                return false;
        if (! (fn.endsWith(".htm", Qt::CaseInsensitive) || fn.endsWith(".html", Qt::CaseInsensitive)) )
                fn += ".html"; // default
         setCurrentFileName(fn);
         return fileSave();
//        QTextDocumentWriter writer(fn);

//        writer.write(ui->textEdit->document());
//            return true;
}

bool MainWindow::fileSaveAs()
{
        QString fn = QFileDialog::getSaveFileName(this, tr("Save as..."),
                                                  QString(), tr("ODF files (*.odt);;HTML-Files (*.htm *.html);;All Files (*)"));
        if (fn.isEmpty())
                return false;
        if (! (fn.endsWith(".odt", Qt::CaseInsensitive) || fn.endsWith(".htm", Qt::CaseInsensitive) || fn.endsWith(".html", Qt::CaseInsensitive)) )
                fn += ".odt"; // default
        // setCurrentFileName(fn);
        // return fileSave();
        QTextDocumentWriter writer(fn);

        writer.write(ui->textEdit->document());

return true;
}

void MainWindow::filePrint()
{
#ifndef QT_NO_PRINTER
        QPrinter printer(QPrinter::HighResolution);
        QPrintDialog *dlg = new QPrintDialog(&printer, this);
        if (ui->textEdit->textCursor().hasSelection())
                dlg->addEnabledOption(QAbstractPrintDialog::PrintSelection);
        dlg->setWindowTitle(tr("Print Document"));
        if (dlg->exec() == QDialog::Accepted) {
                ui->textEdit->print(&printer);
        }
        delete dlg;
#endif
}

void MainWindow::filePrintPreview()
{
#ifndef QT_NO_PRINTER
        QPrinter printer(QPrinter::HighResolution);
        QPrintPreviewDialog preview(&printer, this);
        connect(&preview, SIGNAL(paintRequested(QPrinter*)), SLOT(printPreview(QPrinter*)));
        preview.exec();
#endif
}

void MainWindow::printPreview(QPrinter *printer)
{
#ifdef QT_NO_PRINTER
        Q_UNUSED(printer);
#else
        ui->textEdit->print(printer);
#endif
}


void MainWindow::filePrintPdf()
{
#ifndef QT_NO_PRINTER
        //! [0]
        QString fileName = QFileDialog::getSaveFileName(this, "Export PDF",
                                                        QString(), "*.pdf");
        if (!fileName.isEmpty()) {
                if (QFileInfo(fileName).suffix().isEmpty())
                        fileName.append(".pdf");
                QPrinter printer(QPrinter::HighResolution);
                printer.setOutputFormat(QPrinter::PdfFormat);
                printer.setOutputFileName(fileName);
                ui->textEdit->document()->print(&printer);
        }
        //! [0]
#endif
}

void MainWindow::textColor()
{
        QColor col = QColorDialog::getColor(ui->textEdit->textColor(), this);
        if (!col.isValid())
                return;
        QTextCharFormat fmt;
        fmt.setForeground(col);
        mergeFormatOnWordOrSelection(fmt);
        colorChanged(col);
}
void MainWindow::mergeFormatOnWordOrSelection(const QTextCharFormat &format)
{
        QTextCursor cursor = ui->textEdit->textCursor();
        if (!cursor.hasSelection())
                cursor.select(QTextCursor::WordUnderCursor);
        cursor.mergeCharFormat(format);
        ui->textEdit->mergeCurrentCharFormat(format);
}
void MainWindow::colorChanged(const QColor &c)
{
        QPixmap pix(16, 16);
        pix.fill(c);
        ui->actionColor->setIcon(pix);
}
void MainWindow::textStyle(int styleIndex)
{
        QTextCursor cursor = ui->textEdit->textCursor();

        if (styleIndex != 0) {
                QTextListFormat::Style style = QTextListFormat::ListDisc;

                switch (styleIndex) {
                default:
                case 1:
                        style = QTextListFormat::ListDisc;
                        break;
                case 2:
                        style = QTextListFormat::ListCircle;
                        break;
                case 3:
                        style = QTextListFormat::ListSquare;
                        break;
                case 4:
                        style = QTextListFormat::ListDecimal;
                        break;
                case 5:
                        style = QTextListFormat::ListLowerAlpha;
                        break;
                case 6:
                        style = QTextListFormat::ListUpperAlpha;
                        break;
                case 7:
                        style = QTextListFormat::ListLowerRoman;
                        break;
                case 8:
                        style = QTextListFormat::ListUpperRoman;
                        break;
                }

                cursor.beginEditBlock();

                QTextBlockFormat blockFmt = cursor.blockFormat();

                QTextListFormat listFmt;

                if (cursor.currentList()) {
                        listFmt = cursor.currentList()->format();
                } else {
                        listFmt.setIndent(blockFmt.indent() + 1);
                        blockFmt.setIndent(0);
                        cursor.setBlockFormat(blockFmt);
                }

                listFmt.setStyle(style);

                cursor.createList(listFmt);

                cursor.endEditBlock();
        } else {
                // ####
                QTextBlockFormat bfmt;
                bfmt.setObjectIndex(-1);
                cursor.mergeBlockFormat(bfmt);
        }
}
void MainWindow::textFamily(const QString &f)
{
        QTextCharFormat fmt;
        fmt.setFontFamily(f);
        mergeFormatOnWordOrSelection(fmt);
}
void MainWindow::textBold()
{
        QTextCharFormat fmt;
        fmt.setFontWeight(ui->actionBold->isChecked() ? QFont::Bold : QFont::Normal);
        mergeFormatOnWordOrSelection(fmt);
}

void MainWindow::textUnderline()
{
        QTextCharFormat fmt;
        fmt.setFontUnderline(ui->actionUnderline->isChecked());
        mergeFormatOnWordOrSelection(fmt);
}

void MainWindow::textItalic()
{
        QTextCharFormat fmt;
        fmt.setFontItalic(ui->actionItalic->isChecked());
        mergeFormatOnWordOrSelection(fmt);
}
void MainWindow::textSize(const QString &p)
{
        qreal pointSize = p.toFloat();
        if (p.toFloat() > 0) {
                QTextCharFormat fmt;
                fmt.setFontPointSize(pointSize);
                mergeFormatOnWordOrSelection(fmt);
        }
}
void MainWindow::textAlign(QAction *a)
{
        if (a == actionAlignLeft)
                ui->textEdit->setAlignment(Qt::AlignLeft | Qt::AlignAbsolute);
        else if (a == actionAlignCenter)
                ui->textEdit->setAlignment(Qt::AlignHCenter);
        else if (a == actionAlignRight)
                ui->textEdit->setAlignment(Qt::AlignRight | Qt::AlignAbsolute);
        else if (a == actionAlignJustify)
                ui->textEdit->setAlignment(Qt::AlignJustify);
}

void MainWindow::currentCharFormatChanged(const QTextCharFormat &format)
{
        fontChanged(format.font());
        colorChanged(format.foreground().color());
}

void MainWindow::cursorPositionChanged()
{
        alignmentChanged(ui->textEdit->alignment());
}

void MainWindow::clipboardDataChanged()
{
#ifndef QT_NO_CLIPBOARD
        if (const QMimeData *md = QApplication::clipboard()->mimeData())
                ui->actionPaste->setEnabled(md->hasText());
#endif
}
void MainWindow::fontChanged(const QFont &f)
{
        comboFont->setCurrentIndex(comboFont->findText(QFontInfo(f).family()));
        comboSize->setCurrentIndex(comboSize->findText(QString::number(f.pointSize())));
        ui->actionBold  ->setChecked(f.bold());
        ui->actionItalic  ->setChecked(f.italic());
        ui->actionUnderline->setChecked(f.underline());
}

void MainWindow::alignmentChanged(Qt::Alignment a)
{
        if (a & Qt::AlignLeft) {
                actionAlignLeft->setChecked(true);
        } else if (a & Qt::AlignHCenter) {
                actionAlignCenter->setChecked(true);
        } else if (a & Qt::AlignRight) {
                actionAlignRight->setChecked(true);
        } else if (a & Qt::AlignJustify) {
                actionAlignJustify->setChecked(true);
        }
}

void MainWindow::on_actionOpen_triggered()
{
        load("");
}

void MainWindow::setTextQuran(const QString &txt)
{
        QString fontfamily=comboFont->currentText();
        int fontSize=comboSize->currentText().toInt();
        bool italic=ui->actionItalic ->isChecked();
        bool bold=ui->actionBold ->isChecked();
        bool underline=ui->actionUnderline->isChecked();
        //#######
        QString txte=txt.section("|",0,0);
        QString sura=txt.section("|",1,1);
        QString aya=txt.section("|",2,2);
        QString htm=QString(trUtf8(" <span style=\" font-family:'Amiri Quran'; font-size:%4pt; \">﴿ %1 ﴾ [%2 : %3]</span> </p> "))
                        .arg(txte).arg(sura).arg(aya).arg(comboSize->currentText());
        //   ui->textEditQuran->setHtml( "<span style=\" font-family:'AGA Arabesque,Courier New,courier';\">\357\201\225</span></p>");

        ui->textEdit->insertHtml( htm);
         //#######
         QTextCharFormat fmt;
        fmt.setFontFamily(fontfamily);
        fmt.setFontPointSize(fontSize);
        fmt.setFontUnderline(underline);
        fmt.setFontItalic(italic);
        fmt.setFontWeight(bold ? QFont::Bold : QFont::Normal);
        mergeFormatOnWordOrSelection(fmt);

}
void MainWindow::setTextSymbole(const QString &txt)
{
      //  QFont font=ui->textEdit->font();
       QString fontfamily=comboFont->currentText();
       int fontSize=comboSize->currentText().toInt();
       bool italic=ui->actionItalic ->isChecked();
       bool bold=ui->actionBold ->isChecked();
       bool underline=ui->actionUnderline->isChecked();
 //#######
        QString htm=QString(trUtf8("<span style=\" font-family:'AGA Arabesque'; font-size:%2pt;\">%1</span> </p> "))
                        .arg(txt).arg(comboSize->currentText());



        ui->textEdit->insertHtml( htm);
 //#######
        QTextCharFormat fmt;
//        fmt.setFontFamily(QFontInfo(font).family());
//        fmt.setFontPointSize(font.pointSize());
        fmt.setFontFamily(fontfamily);
        fmt.setFontPointSize(fontSize);
        fmt.setFontUnderline(underline);
        fmt.setFontItalic(italic);
        fmt.setFontWeight(bold ? QFont::Bold : QFont::Normal);
        mergeFormatOnWordOrSelection(fmt);

       // fontChanged(font);


//        ui->actionBold  ->setChecked(f.bold());
//        ui->actionItalic  ->setChecked(f.italic());
//        ui->actionUnderline->setChecked(f.underline());
}


void MainWindow::on_textEdit_highlighted(const QString &arg1)
{
        ui->statusBar->showMessage(arg1);
}

void MainWindow::on_textEdit_anchorClicked(const QUrl &arg1)
{
        QString txt=arg1.toString();
        QString file=txt.section("|",1,1);
        int value=txt.section("|",0,0).toInt();
        if(m_MediaObject.currentSource().fileName()!=file)
                mediaSetSource(file);

        m_MediaObject.seek(value);
        m_MediaObject.play();
}
void MainWindow::addBokmark()
{
        QString fontfamily=comboFont->currentText();
        int fontSize=comboSize->currentText().toInt();
        bool italic=ui->actionItalic ->isChecked();
        bool bold=ui->actionBold ->isChecked();
        bool underline=ui->actionUnderline->isChecked();
  //#######
        //   "<a href=\"0|\">[00:00] </span></a>"
        int value=m_MediaObject.currentTime();
        int sec = value/1000;
        int min = sec/60;
        int hour = min/60;
        QTime playTime(hour%60, min%60, sec%60);
        QString timeFormat = "m:ss";
        if (hour > 0)
                timeFormat = "h:mm:ss";
        QString timeString = playTime.toString(timeFormat);


        QString file=m_MediaObject.currentSource().fileName();
        QString htm=QString("<a href=\"%1|%2\">[%3]</span></a>")
                        .arg(QString::number(value)).arg(file).arg(timeString);

        ui->textEdit->insertHtml(htm);

        QTextCharFormat fmt;
       fmt.setFontFamily(fontfamily);
       fmt.setFontPointSize(fontSize);
       fmt.setFontUnderline(underline);
       fmt.setFontItalic(italic);
       fmt.setFontWeight(bold ? QFont::Bold : QFont::Normal);
       mergeFormatOnWordOrSelection(fmt);
}
void MainWindow::aboutThisProgram()
{
        DialogAbout *dlg=new DialogAbout(this);
        dlg->exec();
}
